package com.yunzhi.demo.common;

import java.io.Serializable;
import java.util.Map;

/**
 * 接口统一状态返回BEAN.
 */
public class ResponseBean<T> implements Serializable {

    private static final long serialVersionUID = 3593827217136880822L;

    public static final int CODE_SUCCESS = 1000;
    public static final int ERROR_AUTH_FAIL = 1001;
    public static final int ERROR_AUTH_EXPIRED = 1002;
    public static final int ERROR_MISSING_PARAMS = 1003;
    public static final int ERROR_ILLEGAL_PARAMS = 1004;
    public static final int ERROR_UNAVAILABLE = 1005;

    private int code;
    private String message;
    private T data;
    private String token;

    /**
     * 禁止外部 new
     */
    private ResponseBean() {
        code = ResponseBean.CODE_SUCCESS;
    }

    public static <T> ResponseBean success(T data, String ...msgs) {
        ResponseBean responseBean = new ResponseBean();
        responseBean.code = ResponseBean.CODE_SUCCESS;
        responseBean.data = data;
        responseBean.token = getContextToken();

        if (null != msgs && msgs.length > 0) {
            responseBean.message = msgs[0];
        }

        return responseBean;
    }

    private static String getContextToken() {
        Map<String, Object> tokenParam = CurrentContext.getTokenParam();
        return null != tokenParam ? (String) tokenParam.get("token") : null;
    }

    public static <T> ResponseBean error(String msg, int code, T ...datas) {
        ResponseBean responseBean = new ResponseBean();
        responseBean.code = code;
        responseBean.message = msg;

        if (null != datas && datas.length > 0) {
            responseBean.data = datas[0];
        }

        return responseBean;
    }

    public int getCode() {
        return code;
    }

    public String getMessage() {
        return message;
    }

    public T getData() {
        return data;
    }

    public String getToken() {
        return token;
    }
    
    @Override
    public String toString() {
        return  "{ code: "+code+", message: "+message+", data: <T> }";
    }
}
