package com.yunzhi.demo.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.yunzhi.demo.common.BaseController;
import com.yunzhi.demo.common.ResponseBean;
import com.yunzhi.demo.entity.TaPost;
import com.yunzhi.demo.service.ITaPersonService;
import com.yunzhi.demo.service.ITaPostDataService;
import com.yunzhi.demo.service.ITaPostService;
import com.yunzhi.demo.service.ITaReadLogService;
import com.yunzhi.demo.vo.StatisPerson;
import com.yunzhi.demo.vo.StatisPost;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;
import java.util.Map;


@Api(tags = "查询统计")
@RestController
public class StatisticController extends BaseController {

    @Autowired
    ITaPostDataService iTaPostDataService;

    @Autowired
    ITaPersonService iTaPersonService;

    @Autowired
    ITaReadLogService iTaReadLogService;

    @Autowired
    ITaPostService iTaPostService;


    @GetMapping("/admin/index/post-data")
    @ApiOperation(value="首页简单的几个统计", notes = "首页简单的几个统计", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean getIndexPostData() {
        // 获取总科普数, 以及 浏览量
        Map<String, Object> result = iTaPostDataService.getIndexBasicData();
        // 获取总学生人员
        int totalStudents = iTaPersonService.countStudent();
        result.put("student", totalStudents);

        return ResponseBean.success(result);
    }

    /**
     * 首页科普浏览统计
     * @param startDate
     * @param endDate
     * @return
     */
    @GetMapping("/admin/index/post-pv")
    @ApiOperation(value="首页科普浏览统计", notes = "首页科普浏览统计", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean getIndexPostPV(@ApiParam(value = "开始日期", example="YYYY-MM-DD") @RequestParam("startDate") String startDate,
                                       @ApiParam(value = "结束日期", example="YYYY-MM-DD") @RequestParam("endDate") String endDate) {
        List<Map<String, Integer>> result = iTaReadLogService.getIndexPostPV(startDate, endDate);
        return ResponseBean.success(result);
    }

    /**
     * 首页学生浏览统计
     * @param startDate
     * @param endDate
     * @return
     */
    @GetMapping("/admin/index/post-uv")
    public ResponseBean getIndexPostUV(@ApiParam(value = "开始日期", example="YYYY-MM-DD") @RequestParam("startDate") String startDate,
                                       @ApiParam(value = "结束日期", example="YYYY-MM-DD") @RequestParam("endDate") String endDate) {
        List<Map<String, Integer>> result = iTaReadLogService.getIndexPostUV(startDate, endDate);
        return ResponseBean.success(result);
    }

    @GetMapping("/admin/statis/post-data")
    @ApiOperation(value="科普统计", notes = "科普统计", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean getPostData(@ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
                                    @ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize,
                                    @ApiParam(value = "开始日期", example="YYYY-MM-DD") @RequestParam(value = "startDate", required = false) String startDate,
                                    @ApiParam(value = "结束日期", example="YYYY-MM-DD") @RequestParam(value = "endDate", required = false) String endDate,
                                    @ApiParam(value = "科普分类") @RequestParam(value = "name", required = false) String name,
                                    @ApiParam(value = "科普分类") @RequestParam(value = "typeId", required = false) String typeId,
                                    @ApiParam("升序") @RequestParam(value = "asc", required = false) String asc,
                                    @ApiParam("降序") @RequestParam(value = "desc", required = false) String desc) {
        IPage<StatisPost> pg = new Page<>(pageNum, pageSize);

        IPage<StatisPost> postList = iTaPostService.getPostStatis(pg, startDate, endDate, name, typeId, asc, desc);

        return ResponseBean.success(postList);
    }

    @GetMapping("/admin/statis/student-data")
    @ApiOperation(value="学生统计", notes = "学生统计", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean getPersonData(@ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
                                      @ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize,
                                      @ApiParam(value = "姓名") @RequestParam(value = "name", required = false) String name,
                                      @ApiParam(value = "学校") @RequestParam(value = "schoolId", required = false) String schoolId,
                                      @ApiParam(value = "专业") @RequestParam(value = "specialtyId", required = false) String specialtyId,
                                      @ApiParam("升序") @RequestParam(value = "asc", required = false) String asc,
                                      @ApiParam("降序") @RequestParam(value = "desc", required = false) String desc) {
        IPage<StatisPerson> pg = new Page<>(pageNum, pageSize);

        IPage<StatisPerson> personList = iTaPersonService.getStudentStatis(pg, name, schoolId, specialtyId, asc, desc);

        return ResponseBean.success(personList);
    }
}
