package com.yunzhi.demo.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.yunzhi.demo.common.*;
import com.yunzhi.demo.entity.*;
import com.yunzhi.demo.service.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
    * 用户 前端控制器
    * </p>
 *
 * @author yansen
 * @since 2021-04-15
 */

@Api(tags = "用户")
@RestController
@RequestMapping("/")
public class SysUserController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(SysUserController.class);

    @Autowired
    public ISysUserService iSysUserService;

    @Autowired
    public ISysUserRoleService iSysUserRoleService;

    @Autowired
    public ISysMenuService iSysMenuService;

    @Autowired
    public ISysLoginService iSysLoginService;

    @Autowired
    public ISysRoleService iSysRoleService;


    /**
     * 分页查询列表
     * @param pageNum
     * @param pageSize
     * @return
     */
    @RequestMapping(value="/admin/user",method= RequestMethod.GET)
    @ApiOperation(value="列表", notes = "列表", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean sysUserList(@ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
                                    @ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize,
                                    @ApiParam("姓名") @RequestParam(value ="name", required = false) String name,
                                    @ApiParam("手机") @RequestParam(value ="phone", required = false) String phone,
                                    @ApiParam("角色") @RequestParam(value ="roleId", required = false) Integer roleId) throws Exception{

		    IPage<SysUser> pg = new Page<>(pageNum, pageSize);
            IPage<SysUser> result = iSysUserService.getPageListBy(pg, name, phone, roleId);
            return ResponseBean.success(result);
    }

    /**
     * 保存对象
     * @param sysUser 实体对象
     * @return
     */
    @RequestMapping(value="/admin/user",method= RequestMethod.POST)
    @ApiOperation(value="保存", notes = "保存", httpMethod = "POST", response = ResponseBean.class)
    public ResponseBean sysUserAdd(@ApiParam("保存内容") @RequestBody SysUser sysUser) throws Exception{
//        if (StringUtils.isEmpty(sysUser.getPhone())) {
//            throw new Exception("电话不能为空");
//        }
        if (StringUtils.isEmpty(sysUser.getUserName())) {
            throw new Exception("姓名不能为空");
        }
        if (StringUtils.isEmpty(sysUser.getLoginName())) {
            throw new Exception("登录名不能为空");
        }

        if (!StringUtils.isEmpty(sysUser.getPhone())) {
            SysUser user1 = iSysUserService.getByProp("phone", sysUser.getPhone());
            if (null != user1) {
                throw new Exception("手机号码重复");
            }
        }

        SysLogin sysLogin = iSysLoginService.getByLoginName(sysUser.getLoginName());
        if (null != sysLogin) {
            throw new Exception("登录名重复");
        }

        if (iSysUserService.save(sysUser)){
            SysLogin newLogin = new SysLogin();
            newLogin.setUserId(sysUser.getUserId());
            newLogin.setLoginName(sysUser.getLoginName());
            newLogin.setPassword(EncryptUtils.md5(EncryptUtils.md5(Constants.DEFAULT_PASSWORD), sysUser.getUserId()));

            iSysLoginService.save(newLogin);

            return ResponseBean.success(sysUser);
        }else {
            return ResponseBean.error("保存失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 根据id删除对象
     * @param id  实体ID
     */
    @RequestMapping(value="/admin/user/{id}", method= RequestMethod.DELETE)
    @ApiOperation(value="删除", notes = "删除", httpMethod = "DELETE", response = ResponseBean.class)
    public ResponseBean sysUserDelete(@ApiParam("对象ID") @PathVariable Integer id) throws Exception{
        SysUser sysUser = iSysUserService.getById(id);
        if (null == sysUser || Constants.STATUS_DELETED.equals(sysUser.getStatus())) {
            return ResponseBean.success("success");
        }

        sysUser.setStatus(Constants.STATUS_DELETED);

        if(iSysUserService.updateById(sysUser)){
            return ResponseBean.success("success");
        }else {
            return ResponseBean.error("删除失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 修改对象
     * @param id  实体ID
     * @param sysUser 实体对象
     * @return
     */
    @RequestMapping(value="/admin/user/{id}",method= RequestMethod.PUT)
    @ApiOperation(value="更新", notes = "更新", httpMethod = "PUT", response = ResponseBean.class)
    public ResponseBean sysUserUpdate(@ApiParam("对象ID") @PathVariable String id,
                                      @ApiParam("更新内容") @RequestBody SysUser sysUser) throws Exception{
        if (StringUtils.isEmpty(sysUser.getUserName())) {
            throw new Exception("姓名不能为空");
        }
        if (StringUtils.isEmpty(sysUser.getLoginName())) {
            throw new Exception("登录名不能为空");
        }

        if (!StringUtils.isEmpty(sysUser.getPhone())) {
            SysUser user1 = iSysUserService.getByProp("phone", sysUser.getPhone());
            if (null != user1 && !user1.getUserId().equals(id)) {
                throw new Exception("手机号码重复");
            }
        }

        // 依据登录名查询的登录信息
        // 有可能跟当前人员的不一致, 不一致说明登录名重复
        SysLogin sysLogin = iSysLoginService.getByLoginName(sysUser.getLoginName());
        if (null != sysLogin && !sysLogin.getUserId().equals(id)) {
            throw new Exception("登录名重复");
        }

        if (iSysUserService.updateById(sysUser)){
            // 如果新的登录名不存在, 说明修改了登录名
            if (null == sysLogin) {
                sysLogin = iSysLoginService.getByUser(id);
                sysLogin.setLoginName(sysUser.getLoginName());
                iSysLoginService.updateById(sysLogin);
            }

            return ResponseBean.success(iSysUserService.getById(id));
        }else {
            return ResponseBean.error("修改失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 根据id查询对象
     * @param id  实体ID
     */
    @RequestMapping(value="/admin/user/{id}",method= RequestMethod.GET)
    @ApiOperation(value="详情", notes = "详情", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean sysUserGet(@ApiParam("对象ID") @PathVariable String id) throws Exception{
        SysUser sysUser = iSysUserService.getById(id);
        if (null == sysUser || Constants.STATUS_DELETED.equals(sysUser.getStatus())) {
            throw new Exception("当前人员信息不存在");
        }

        SysLogin sysLogin = iSysLoginService.getByUser(sysUser.getUserId());
        if (null != sysLogin) {
            sysUser.setLoginName(sysLogin.getLoginName());
        }

        List<SysRole> roleList = iSysRoleService.getByUser(sysUser.getUserId());
        if (null != roleList && roleList.size() > 0) {
            sysUser.setRoleList(roleList);
        }

        return ResponseBean.success(sysUser);
    }


    /**
     * 当前用户
     */
    @RequestMapping(value="/admin/currentUser",method= RequestMethod.GET)
    @ApiOperation(value="当前用户", notes = "详情", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean currentUser() throws Exception{
        SysUser sysUser = getCurrentUser();
        List<SysMenu> sysMenuList = iSysMenuService.getListWithRoles();
        List<SysUserRole> sysUserRoleList = iSysUserRoleService.getListByUser(sysUser.getUserId());

        SysLogin sysLogin = iSysLoginService.getByUser(sysUser.getUserId());
        sysUser.setLoginName(sysLogin.getLoginName());

        Map<String, Object> result = new HashMap<String, Object>() {{
            put("user", sysUser);
            put("menus", sysMenuList);
            put("roles", sysUserRoleList);
        }};

        return ResponseBean.success(result);
    }
}
