package com.yunzhi.demo.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.yunzhi.demo.common.BaseController;
import com.yunzhi.demo.common.Constants;
import com.yunzhi.demo.common.ResponseBean;
import com.yunzhi.demo.common.StringUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import com.yunzhi.demo.service.ITaShareSettingService;
import com.yunzhi.demo.entity.TaShareSetting;
import org.springframework.web.bind.annotation.RestController;

/**
 * <p>
    * 分享设置  前端控制器
    * </p>
 *
 * @author yansen
 * @since 2021-04-25
 */

@Api(tags = "分享设置 ")
@RestController
@RequestMapping("/")
public class TaShareSettingController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(TaShareSettingController.class);

    @Autowired
    public ITaShareSettingService iTaShareSettingService;


    /**
     * 保存对象
     * @param taShareSetting 实体对象
     * @return
     */
    @RequestMapping(value="/admin/share-setting",method= RequestMethod.POST)
    @ApiOperation(value="保存", notes = "保存", httpMethod = "POST", response = ResponseBean.class)
    public ResponseBean taShareSettingAdd(@ApiParam("保存内容") @RequestBody TaShareSetting taShareSetting) throws Exception {
        if (StringUtils.isEmpty(taShareSetting.getTargetId())) {
            throw new Exception("未设置分享对象");
        }

        if (StringUtils.isEmpty(taShareSetting.getTargetType())) {
            taShareSetting.setTargetType(Constants.RESOURCE_TYPE_POST);
        }

        if (iTaShareSettingService.save(taShareSetting)){
            return ResponseBean.success(taShareSetting);
        }else {
            return ResponseBean.error("保存失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 修改对象
     * @param id  实体ID
     * @param taShareSetting 实体对象
     * @return
     */
    @RequestMapping(value="/admin/share-setting/{id}",method= RequestMethod.PUT)
    @ApiOperation(value="更新", notes = "更新", httpMethod = "PUT", response = ResponseBean.class)
    public ResponseBean taShareSettingUpdate(@ApiParam("对象ID") @PathVariable Integer id,
                                             @ApiParam("更新内容") @RequestBody TaShareSetting taShareSetting) throws Exception{
        if (StringUtils.isEmpty(taShareSetting.getTargetId())) {
            throw new Exception("未设置分享对象");
        }

        if (StringUtils.isEmpty(taShareSetting.getTargetType())) {
            taShareSetting.setTargetType(Constants.RESOURCE_TYPE_POST);
        }

        taShareSetting.setSerialNo(id);

        if (iTaShareSettingService.updateById(taShareSetting)){
            return ResponseBean.success(iTaShareSettingService.getById(id));
        }else {
            return ResponseBean.error("修改失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

//    /**
//     * 根据id查询对象
//     * @param id  实体ID
//     */
//    @RequestMapping(value="/admin/share-setting/{id}",method= RequestMethod.GET)
//    @ApiOperation(value="详情", notes = "详情", httpMethod = "GET", response = ResponseBean.class)
//    public ResponseBean taShareSettingGet(@ApiParam("对象ID") @PathVariable Integer id) throws Exception{
//        return ResponseBean.success(iTaShareSettingService.getById(id));
//    }
}
