package com.yunzhi.demo.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.yunzhi.demo.common.BaseController;
import com.yunzhi.demo.common.Constants;
import com.yunzhi.demo.common.ResponseBean;
import com.yunzhi.demo.common.StringUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import com.yunzhi.demo.service.ITdPostTypeService;
import com.yunzhi.demo.entity.TdPostType;
import org.springframework.web.bind.annotation.RestController;

/**
 * <p>
    * 文章分类 前端控制器
    * </p>
 *
 * @author yansen
 * @since 2021-04-15
 */

@Api(tags = "文章分类")
@RestController
@RequestMapping("/")
public class TdPostTypeController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(TdPostTypeController.class);

    @Autowired
    public ITdPostTypeService iTdPostTypeService;


    /**
     * 分页查询列表
     * @param pageNum
     * @param pageSize
     * @return
     */
    @RequestMapping(value="/{client}/post-type",method= RequestMethod.GET)
    @ApiOperation(value="列表", notes = "列表", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean tdPostTypeList(@ApiParam("客户端") @PathVariable String client,
                                       @ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
                                       @ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize,
                                       @ApiParam("状态") @RequestParam(value = "status", required = false) Integer status) throws Exception{

		    IPage<TdPostType> pg = new Page<>(pageNum, pageSize);
            QueryWrapper<TdPostType> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq(null != status, "status", status);
            queryWrapper.gt("status", Constants.STATUS_DELETED);
            queryWrapper.orderByDesc("sort_no");
            queryWrapper.orderByDesc("create_date");

            IPage<TdPostType> result = iTdPostTypeService.page(pg, queryWrapper);
            return ResponseBean.success(result);
    }

    /**
     * 保存对象
     * @param tdPostType 实体对象
     * @return
     */
    @RequestMapping(value="/admin/post-type",method= RequestMethod.POST)
    @ApiOperation(value="保存", notes = "保存", httpMethod = "POST", response = ResponseBean.class)
    public ResponseBean tdPostTypeAdd(@ApiParam("保存内容") @RequestBody TdPostType tdPostType) throws Exception{
        if (StringUtils.isEmpty(tdPostType.getName())) {
            return ResponseBean.error("类型名称不能为空", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        if (iTdPostTypeService.save(tdPostType)){
            return ResponseBean.success(tdPostType);
        }else {
            return ResponseBean.error("保存失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 根据id删除对象
     * @param id  实体ID
     */
    @RequestMapping(value="/admin/post-type/{id}", method= RequestMethod.DELETE)
    @ApiOperation(value="删除", notes = "删除", httpMethod = "DELETE", response = ResponseBean.class)
    public ResponseBean tdPostTypeDelete(@ApiParam("对象ID") @PathVariable String id) throws Exception{
        TdPostType tdPostType = iTdPostTypeService.getById(id);
        if (null == tdPostType || Constants.STATUS_DELETED.equals(tdPostType.getStatus())) {
            return ResponseBean.success("success");
        }

        tdPostType.setStatus(Constants.STATUS_DELETED);

        if(iTdPostTypeService.updateById(tdPostType)){
            return ResponseBean.success("success");
        }else {
            return ResponseBean.error("删除失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 修改对象
     * @param id  实体ID
     * @param tdPostType 实体对象
     * @return
     */
    @RequestMapping(value="/admin/post-type/{id}",method= RequestMethod.PUT)
    @ApiOperation(value="更新", notes = "更新", httpMethod = "PUT", response = ResponseBean.class)
    public ResponseBean tdPostTypeUpdate(@ApiParam("对象ID") @PathVariable String id,
                                         @ApiParam("更新内容") @RequestBody TdPostType tdPostType) throws Exception {
        if (StringUtils.isEmpty(tdPostType.getName())) {
            return ResponseBean.error("类型名称不能为空", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        if (iTdPostTypeService.updateById(tdPostType)){
            return ResponseBean.success(iTdPostTypeService.getById(id));
        }else {
            return ResponseBean.error("修改失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 根据id查询对象
     * @param id  实体ID
     */
    @RequestMapping(value="/admin/post-type/{id}",method= RequestMethod.GET)
    @ApiOperation(value="详情", notes = "详情", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean tdPostTypeGet(@ApiParam("对象ID") @PathVariable Integer id) throws Exception{
        TdPostType tdPostType = iTdPostTypeService.getById(id);
        if (null == tdPostType || Constants.STATUS_DELETED.equals(tdPostType.getStatus())) {
            return ResponseBean.error("没有找到对应的信息", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        return ResponseBean.success(tdPostType);
    }
}
