package com.yunzhi.demo.controller;

import cn.binarywang.wx.miniapp.bean.WxMaJscode2SessionResult;
import cn.binarywang.wx.miniapp.bean.WxMaPhoneNumberInfo;
import cn.binarywang.wx.miniapp.bean.WxMaUserInfo;
import com.yunzhi.demo.common.*;
import com.yunzhi.demo.entity.TaPerson;
import com.yunzhi.demo.entity.TaPersonData;
import com.yunzhi.demo.service.ITaPersonDataService;
import com.yunzhi.demo.service.ITaPersonService;
import com.yunzhi.demo.vo.WxMaAuthParam;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.HashMap;
import java.util.Map;


@Api(tags = "微信小程序")
@RestController
@RequestMapping("/ma")
public class WxMaController extends BaseController {

    @Autowired
    WxUtils wxUtils;

    @Autowired
    ITaPersonService iTaPersonService;

    @Autowired
    ITaPersonDataService iTaPersonDataService;

    @PostMapping("/login")
    @ApiOperation(value="登录", notes = "登录", httpMethod = "POST", response = ResponseBean.class)
    public ResponseBean login(@ApiParam("登录参数") @RequestParam String code) throws Exception {
        WxMaJscode2SessionResult info = wxUtils.getService().getUserService().getSessionInfo(code);

        String openid = info.getOpenid();
        TaPerson taPerson = iTaPersonService.getByOpenid(openid);
        if (null == taPerson) {
            taPerson = new TaPerson();
            taPerson.setOpenid(openid);
            iTaPersonService.save(taPerson);

            TaPersonData taPersonData = new TaPersonData();
            taPersonData.setPersonId(taPerson.getPersonId());
            iTaPersonDataService.save(taPersonData);
        }

        Map<String, Object> tokenClaims = new HashMap<>();
        tokenClaims.put("userId", taPerson.getPersonId());
//        tokenClaims.put("sessionKey", info.getSessionKey());
        tokenClaims.put("miniApp", true);
        String token = JWTUtils.encode(tokenClaims);

        Map<String, Object> rtn = new HashMap<>();
        rtn.put("person", taPerson);
        rtn.put("sessionKey", info.getSessionKey());
        rtn.put("token", token);

        return ResponseBean.success(rtn);
    }

    @PutMapping("/auth-user")
    @ApiOperation(value="授权头像", notes = "授权头像", httpMethod = "PUT", response = TaPerson.class)
    public ResponseBean updateUserInfo(@ApiParam("授权头像的参数") @RequestBody WxMaAuthParam params) throws Exception {
        checkAuthParams(params);

        // 解密用户信息
        WxMaUserInfo userInfo = wxUtils.getService().getUserService().getUserInfo(params.getSessionKey(), params.getEncryptedData(), params.getIv());

        TaPerson taPerson = getCurrentPerson();
        if (null == taPerson) {
            throw new Exception("校验当前人员失败, 请重试");
        }

        taPerson.setAvatar(userInfo.getAvatarUrl());
        taPerson.setNickName(userInfo.getNickName());
        taPerson.setSex(Integer.parseInt(userInfo.getGender()));

        iTaPersonService.updateById(taPerson);

        return ResponseBean.success(taPerson);
    }

    @ApiOperation(value="授权手机", notes = "授权手机", httpMethod = "PUT", response = TaPerson.class)
    @PutMapping("/auth-phone")
    public ResponseBean updateUserPhone(@ApiParam("授权手机的参数") @RequestBody WxMaAuthParam params) throws Exception {
//        checkAuthParams(params);

        // 解密
        WxMaPhoneNumberInfo phoneNoInfo = wxUtils.getService().getUserService().getPhoneNoInfo(params.getSessionKey(), params.getEncryptedData(), params.getIv());

        TaPerson taPerson = getCurrentPerson();
        if (null == taPerson) {
            throw new Exception("校验当前人员失败, 请重试");
        }

        taPerson.setPhone(phoneNoInfo.getPhoneNumber());
        iTaPersonService.updateById(taPerson);

        return ResponseBean.success(taPerson);
    }

    private void checkAuthParams(WxMaAuthParam params) throws Exception {
        String signature = params.getSignature();
        String rawData = params.getRawData();
        String encryptedData = params.getEncryptedData();
        String iv = params.getIv();
        String sessionKey = params.getSessionKey();

        if (StringUtils.isEmpty(signature) || StringUtils.isEmpty(rawData) || StringUtils.isEmpty(encryptedData) || StringUtils.isEmpty(iv) || StringUtils.isEmpty(sessionKey)) {
            throw new Exception("缺失授权参数");
        }

        boolean isValid = wxUtils.getService().getUserService().checkUserInfo(sessionKey, rawData, signature);
        if (!isValid) {
            throw new Exception("校验授权参数失败, 请重试");
        }

        return;
    }
}
