package com.yunzhi.demo.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.yunzhi.demo.common.Constants;
import com.yunzhi.demo.common.StringUtils;
import com.yunzhi.demo.entity.SysConfig;
import com.yunzhi.demo.entity.TaPerson;
import com.yunzhi.demo.entity.TaPointsLog;
import com.yunzhi.demo.entity.TaPost;
import com.yunzhi.demo.mapper.SysConfigMapper;
import com.yunzhi.demo.mapper.TaPersonDataMapper;
import com.yunzhi.demo.mapper.TaPointsLogMapper;
import com.yunzhi.demo.service.ITaPointsLogService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;

/**
 * <p>
 * 积分历史 服务实现类
 * </p>
 *
 * @author yansen
 * @since 2021-04-15
 */
@Slf4j
@Service
public class TaPointsLogServiceImpl extends ServiceImpl<TaPointsLogMapper, TaPointsLog> implements ITaPointsLogService {

    @Autowired
    TaPersonDataMapper taPersonDataMapper;

    @Autowired
    SysConfigMapper sysConfigMapper;

    @Async
    @Override
    public void sendPoints(TaPost taPost, TaPerson taPerson) {
        if (null == taPost || null == taPerson) {
            return;
        }

        Integer points = taPost.getPoints();
        if (null == points) {
            points = 0;
        }

        // 先验证是否已经获取过积分
        TaPointsLog taPointsLog = getByPersonPost(taPerson.getPersonId(), taPost.getPostId());
        if (null != taPointsLog) {
            // 已经答过该题, 不处理
            return;
        }

        // 查询积分-学分转换规则
        String ratio = sysConfigMapper.getValueOf(Constants.SYS_PARAM_POINT_CREDIT_RATIO);
        String creditNum = points.toString();

        taPointsLog = new TaPointsLog();
        taPointsLog.setPersonId(taPerson.getPersonId());
        taPointsLog.setTargetType(Constants.RESOURCE_TYPE_POST);
        taPointsLog.setTargetId(taPost.getPostId());
        taPointsLog.setPointsNum(points);
        taPointsLog.setRatio(ratio);
        taPointsLog.setCreditNum(creditNum);

        try {
            save(taPointsLog);

            // 累计入账户
            taPersonDataMapper.updatePoints(taPerson.getPersonId(), points, creditNum);
        } catch (Exception e) {
            e.printStackTrace();

            log.error(String.format("奖励[%s-%s] 阅读[%s-%s] 积分 %d 失败: %s",
                    StringUtils.ifNull(taPerson.getName(), taPerson.getNickName()),
                    taPerson.getPersonId(),
                    taPost.getName(),
                    taPost.getPostId(),
                    points,
                    e.getMessage()));
        }
    }

    private TaPointsLog getByPersonPost(String personId, String targetId) {
        QueryWrapper<TaPointsLog> queryWrapper = new QueryWrapper<TaPointsLog>()
                .eq("person_id", personId)
                .eq("target_id", targetId)
                .eq("status", Constants.STATUS_NORMAL);

        return getOne(queryWrapper);
    }
}
